import React from "react";
import { Button } from "@react-components/button";
import { Popper } from "@react-components/popper";
import { act, render, waitFor } from "@testing-library/react";
import { createRef, forwardRef, useState } from "react";
import { isNil } from "lodash";
import userEvent from "@utils/user-event";
const POPPER_WRAPPER_ID = "popper-wrapper";
const SimplePopper = /*#__PURE__*/forwardRef((props, ref) => {
  const [triggerElement, setTriggerElement] = useState();
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(Button, {
    ref: setTriggerElement
  }, "Open"), !isNil(triggerElement) ? /*#__PURE__*/React.createElement(Popper, Object.assign({}, props, {
    show: true,
    animate: false,
    triggerElement: triggerElement,
    ref: ref
  }), /*#__PURE__*/React.createElement("div", null, "Popper")) : null);
}); // ***** Behaviors *****

test("don't toggle popper when disabled", async () => {
  const {
    getByTestId,
    queryByTestId
  } = render( /*#__PURE__*/React.createElement(SimplePopper, {
    disabled: true
  }));
  act(() => {
    userEvent.click(getByTestId("button"));
  });
  expect(queryByTestId(POPPER_WRAPPER_ID)).not.toBeInTheDocument();
}); // ***** API *****

test("spread additional props on the root element", async () => {
  const ref = /*#__PURE__*/createRef();
  render( /*#__PURE__*/React.createElement(SimplePopper, {
    ref: ref,
    "data-extra-props-test": "works"
  }));
  await waitFor(() => expect(ref.current.getAttribute("data-extra-props-test")).toBe("works"));
}); // ***** Refs *****

test("when wrapped, ref is a DOM element", async () => {
  const ref = /*#__PURE__*/createRef();
  render( /*#__PURE__*/React.createElement(SimplePopper, {
    ref: ref
  }));
  await waitFor(() => expect(ref.current).not.toBeNull());
  expect(ref.current instanceof HTMLElement).toBeTruthy();
  expect(ref.current.tagName).toBe("DIV");
  expect(ref.current.getAttribute("data-testid")).toBe(POPPER_WRAPPER_ID);
});
test("when wrapped, using a callback ref, ref is a DOM element", async () => {
  let refNode = null;
  render( /*#__PURE__*/React.createElement(SimplePopper, {
    ref: node => {
      refNode = node;
    }
  }));
  await waitFor(() => expect(refNode).not.toBeNull());
  expect(refNode instanceof HTMLElement).toBeTruthy();
  expect(refNode.tagName).toBe("DIV");
  expect(refNode.getAttribute("data-testid")).toBe(POPPER_WRAPPER_ID);
});
test("when wrapped, set ref once", async () => {
  const handler = jest.fn();
  render( /*#__PURE__*/React.createElement(SimplePopper, {
    ref: handler
  }));
  await waitFor(() => expect(handler).toHaveBeenCalledTimes(1));
});
test("when not wrapped, ref is a DOM element", async () => {
  const ref = /*#__PURE__*/createRef();
  render( /*#__PURE__*/React.createElement(SimplePopper, {
    noWrap: true,
    ref: ref
  }));
  await waitFor(() => expect(ref.current).not.toBeNull());
  expect(ref.current instanceof HTMLElement).toBeTruthy();
  expect(ref.current.tagName).toBe("DIV");
  expect(ref.current.getAttribute("data-testid")).not.toBe(POPPER_WRAPPER_ID);
});
test("when not wrapped and using a callback ref, ref is a DOM element", async () => {
  let refNode = null;
  render( /*#__PURE__*/React.createElement(SimplePopper, {
    noWrap: true,
    ref: node => {
      refNode = node;
    }
  }));
  await waitFor(() => expect(refNode).not.toBeNull());
  expect(refNode instanceof HTMLElement).toBeTruthy();
  expect(refNode.tagName).toBe("DIV");
  expect(refNode.getAttribute("data-testid")).not.toBe(POPPER_WRAPPER_ID);
});
test("when not wrapped, set ref once", async () => {
  const handler = jest.fn();
  render( /*#__PURE__*/React.createElement(SimplePopper, {
    noWrap: true,
    ref: handler
  }));
  await waitFor(() => expect(handler).toHaveBeenCalledTimes(1));
});